@@ -27,15 +27,7 @@ ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 POSSIBILITY OF SUCH DAMAGE.
 */
 
-#include <mosquitto.h>
-#include <logging_mosq.h>
-#include <messages_mosq.h>
-#include <mqtt3_protocol.h>
-#include <net_mosq.h>
-#include <read_handle.h>
-#include <send_mosq.h>
-#include <util_mosq.h>
-
+#include <assert.h>
 #include <errno.h>
 #include <stdio.h>
 #include <string.h>
@@ -46,6 +38,20 @@ POSSIBILITY OF SUCH DAMAGE.
 typedef int ssize_t;
 #endif
 
+#include <mosquitto.h>
+#include <logging_mosq.h>
+#include <messages_mosq.h>
+#include <memory_mosq.h>
+#include <mqtt3_protocol.h>
+#include <net_mosq.h>
+#include <read_handle.h>
+#include <send_mosq.h>
+#include <util_mosq.h>
+
+#ifndef ECONNRESET
+#define ECONNRESET 104
+#endif
+
 void mosquitto_lib_version(int *major, int *minor, int *revision)
 {
 	if(major) *major = LIBMOSQUITTO_MAJOR;
@@ -60,7 +66,7 @@ int mosquitto_lib_init(void)
 	WSAStartup(MAKEWORD(2,2), &wsaData);
 #endif
 
-	return 0;
+	return MOSQ_ERR_SUCCESS;
 }
 
 int mosquitto_lib_cleanup(void)
@@ -69,7 +75,7 @@ int mosquitto_lib_cleanup(void)
 	WSACleanup();
 #endif
 
-	return 0;
+	return MOSQ_ERR_SUCCESS;
 }
 
 struct mosquitto *mosquitto_new(const char *id, void *obj)
@@ -78,27 +84,29 @@ struct mosquitto *mosquitto_new(const char *id, void *obj)
 
 	if(!id) return NULL;
 
-	mosq = (struct mosquitto *)calloc(1, sizeof(struct mosquitto));
+	mosq = (struct mosquitto *)_mosquitto_calloc(1, sizeof(struct mosquitto));
 	if(mosq){
 		if(obj){
 			mosq->obj = obj;
 		}else{
 			mosq->obj = mosq;
 		}
-		mosq->sock = INVALID_SOCKET;
-		mosq->keepalive = 60;
+		mosq->core.sock = INVALID_SOCKET;
+		mosq->core.keepalive = 60;
 		mosq->message_retry = 20;
 		mosq->last_retry_check = 0;
-		mosq->id = strdup(id);
-		mosq->in_packet.payload = NULL;
-		_mosquitto_packet_cleanup(&mosq->in_packet);
-		mosq->out_packet = NULL;
-		mosq->last_msg_in = time(NULL);
-		mosq->last_msg_out = time(NULL);
-		mosq->last_mid = 0;
-		mosq->state = mosq_cs_new;
+		mosq->core.id = _mosquitto_strdup(id);
+		mosq->core.username = NULL;
+		mosq->core.password = NULL;
+		mosq->core.in_packet.payload = NULL;
+		_mosquitto_packet_cleanup(&mosq->core.in_packet);
+		mosq->core.out_packet = NULL;
+		mosq->core.last_msg_in = time(NULL);
+		mosq->core.last_msg_out = time(NULL);
+		mosq->core.last_mid = 0;
+		mosq->core.state = mosq_cs_new;
 		mosq->messages = NULL;
-		mosq->will = NULL;
+		mosq->core.will = NULL;
 		mosq->on_connect = NULL;
 		mosq->on_publish = NULL;
 		mosq->on_message = NULL;
@@ -112,60 +120,121 @@ struct mosquitto *mosquitto_new(const char *id, void *obj)
 
 int mosquitto_will_set(struct mosquitto *mosq, bool will, const char *topic, uint32_t payloadlen, const uint8_t *payload, int qos, bool retain)
 {
-	if(!mosq) return 1;
-	if(will && !topic) return 1;
+	int rc = MOSQ_ERR_SUCCESS;
 
-	if(mosq->will){
-		if(mosq->will->topic) free(mosq->will->topic);
-		if(mosq->will->payload){
-			free(mosq->will->payload);
-			mosq->will->payload = NULL;
+	if(!mosq || (will && !topic)) return MOSQ_ERR_INVAL;
+
+	if(mosq->core.will){
+		if(mosq->core.will->topic){
+			_mosquitto_free(mosq->core.will->topic);
+			mosq->core.will->topic = NULL;
 		}
-		free(mosq->will);
-		mosq->will = NULL;
+		if(mosq->core.will->payload){
+			_mosquitto_free(mosq->core.will->payload);
+			mosq->core.will->payload = NULL;
+		}
+		_mosquitto_free(mosq->core.will);
+		mosq->core.will = NULL;
 	}
 
 	if(will){
-		mosq->will = calloc(1, sizeof(struct mosquitto_message));
-		if(!mosq->will) return 1;
-		mosq->will->topic = strdup(topic);
-		if(!mosq->will->topic) return 1;
-		mosq->will->payloadlen = payloadlen;
-		if(mosq->will->payloadlen > 0){
-			if(!payload) return 1;
-			mosq->will->payload = malloc(sizeof(uint8_t)*mosq->will->payloadlen);
-			if(!mosq->will->payload) return 1;
-
-			memcpy(mosq->will->payload, payload, payloadlen);
+		mosq->core.will = _mosquitto_calloc(1, sizeof(struct mosquitto_message));
+		if(!mosq->core.will) return MOSQ_ERR_NOMEM;
+		mosq->core.will->topic = _mosquitto_strdup(topic);
+		if(!mosq->core.will->topic){
+			rc = MOSQ_ERR_NOMEM;
+			goto cleanup;
 		}
-		mosq->will->qos = qos;
-		mosq->will->retain = retain;
+		mosq->core.will->payloadlen = payloadlen;
+		if(mosq->core.will->payloadlen > 0){
+			if(!payload){
+				rc = MOSQ_ERR_INVAL;
+				goto cleanup;
+			}
+			mosq->core.will->payload = _mosquitto_malloc(sizeof(uint8_t)*mosq->core.will->payloadlen);
+			if(!mosq->core.will->payload){
+				rc = MOSQ_ERR_NOMEM;
+				goto cleanup;
+			}
+
+			memcpy(mosq->core.will->payload, payload, payloadlen);
+		}
+		mosq->core.will->qos = qos;
+		mosq->core.will->retain = retain;
 	}
 
-	return 0;
+	return MOSQ_ERR_SUCCESS;
+
+cleanup:
+	if(mosq->core.will){
+		if(mosq->core.will->topic) _mosquitto_free(mosq->core.will->topic);
+		if(mosq->core.will->payload) _mosquitto_free(mosq->core.will->payload);
+	}
+	_mosquitto_free(mosq->core.will);
+	mosq->core.will = NULL;
+
+	return rc;
 }
 
+int mosquitto_username_pw_set(struct mosquitto *mosq, const char *username, const char *password)
+{
+	if(!mosq) return MOSQ_ERR_INVAL;
+
+	if(username){
+		mosq->core.username = _mosquitto_strdup(username);
+		if(!mosq->core.username) return MOSQ_ERR_NOMEM;
+		if(mosq->core.password){
+			_mosquitto_free(mosq->core.password);
+			mosq->core.password = NULL;
+		}
+		if(password){
+			mosq->core.password = _mosquitto_strdup(password);
+			if(!mosq->core.password){
+				_mosquitto_free(mosq->core.username);
+				mosq->core.username = NULL;
+				return MOSQ_ERR_NOMEM;
+			}
+		}
+	}else{
+		if(mosq->core.username){
+			_mosquitto_free(mosq->core.username);
+			mosq->core.username = NULL;
+		}
+		if(mosq->core.password){
+			_mosquitto_free(mosq->core.password);
+			mosq->core.password = NULL;
+		}
+	}
+	return MOSQ_ERR_SUCCESS;
+}
+
+
 void mosquitto_destroy(struct mosquitto *mosq)
 {
-	if(mosq->id) free(mosq->id);
+	if(mosq->core.id) _mosquitto_free(mosq->core.id);
 	_mosquitto_message_cleanup_all(mosq);
-	free(mosq);
+	if(mosq->core.will){
+		if(mosq->core.will->topic) _mosquitto_free(mosq->core.will->topic);
+		if(mosq->core.will->payload) _mosquitto_free(mosq->core.will->payload);
+	}
+	_mosquitto_free(mosq->core.will);
+	_mosquitto_free(mosq);
 }
 
 int mosquitto_socket(struct mosquitto *mosq)
 {
-	if(!mosq) return INVALID_SOCKET;
-
-	return mosq->sock;
+	if(!mosq) return MOSQ_ERR_INVAL;
+	return mosq->core.sock;
 }
 
 int mosquitto_connect(struct mosquitto *mosq, const char *host, int port, int keepalive, bool clean_session)
 {
-	if(!mosq || !host || !port) return 1;
+	if(!mosq) return MOSQ_ERR_INVAL;
+	if(!host || !port) return MOSQ_ERR_INVAL;
 
-	mosq->sock = _mosquitto_socket_connect(host, port);
+	mosq->core.sock = _mosquitto_socket_connect(host, port);
 
-	if(mosq->sock == INVALID_SOCKET){
+	if(mosq->core.sock == INVALID_SOCKET){
 		return 1;
 	}
 
@@ -174,9 +243,10 @@ int mosquitto_connect(struct mosquitto *mosq, const char *host, int port, int ke
 
 int mosquitto_disconnect(struct mosquitto *mosq)
 {
-	if(!mosq || mosq->sock == INVALID_SOCKET) return 1;
+	if(!mosq) return MOSQ_ERR_INVAL;
+	if(mosq->core.sock == INVALID_SOCKET) return MOSQ_ERR_NO_CONN;
 
-	mosq->state = mosq_cs_disconnecting;
+	mosq->core.state = mosq_cs_disconnecting;
 
 	return _mosquitto_send_disconnect(mosq);
 }
@@ -186,9 +256,9 @@ int mosquitto_publish(struct mosquitto *mosq, uint16_t *mid, const char *topic,
 	struct mosquitto_message_all *message;
 	uint16_t local_mid;
 
-	if(!mosq || !topic || qos<0 || qos>2) return 1;
+	if(!mosq || !topic || qos<0 || qos>2) return MOSQ_ERR_INVAL;
 
-	local_mid = _mosquitto_mid_generate(mosq);
+	local_mid = _mosquitto_mid_generate(&mosq->core);
 	if(mid){
 		*mid = local_mid;
 	}
@@ -196,8 +266,8 @@ int mosquitto_publish(struct mosquitto *mosq, uint16_t *mid, const char *topic,
 	if(qos == 0){
 		return _mosquitto_send_publish(mosq, local_mid, topic, payloadlen, payload, qos, retain, false);
 	}else{
-		message = calloc(1, sizeof(struct mosquitto_message_all));
-		if(!message) return 1;
+		message = _mosquitto_calloc(1, sizeof(struct mosquitto_message_all));
+		if(!message) return MOSQ_ERR_NOMEM;
 
 		message->next = NULL;
 		message->timestamp = time(NULL);
@@ -208,17 +278,17 @@ int mosquitto_publish(struct mosquitto *mosq, uint16_t *mid, const char *topic,
 			message->state = mosq_ms_wait_pubrec;
 		}
 		message->msg.mid = local_mid;
-		message->msg.topic = strdup(topic);
+		message->msg.topic = _mosquitto_strdup(topic);
 		if(!message->msg.topic){
 			_mosquitto_message_cleanup(&message);
-			return 1;
+			return MOSQ_ERR_NOMEM;
 		}
 		if(payloadlen){
 			message->msg.payloadlen = payloadlen;
-			message->msg.payload = malloc(payloadlen*sizeof(uint8_t));
+			message->msg.payload = _mosquitto_malloc(payloadlen*sizeof(uint8_t));
 			if(!message){
 				_mosquitto_message_cleanup(&message);
-				return 1;
+				return MOSQ_ERR_NOMEM;
 			}
 			memcpy(message->msg.payload, payload, payloadlen*sizeof(uint8_t));
 		}else{
@@ -229,24 +299,23 @@ int mosquitto_publish(struct mosquitto *mosq, uint16_t *mid, const char *topic,
 		message->msg.retain = retain;
 		message->dup = false;
 
-		if(_mosquitto_message_queue(mosq, message)){
-			_mosquitto_message_cleanup(&message);
-			return 1;
-		}
+		_mosquitto_message_queue(mosq, message);
 		return _mosquitto_send_publish(mosq, message->msg.mid, message->msg.topic, message->msg.payloadlen, message->msg.payload, message->msg.qos, message->msg.retain, message->dup);
 	}
 }
 
 int mosquitto_subscribe(struct mosquitto *mosq, uint16_t *mid, const char *sub, int qos)
 {
-	if(!mosq || mosq->sock == INVALID_SOCKET) return 1;
+	if(!mosq) return MOSQ_ERR_INVAL;
+	if(mosq->core.sock == INVALID_SOCKET) return MOSQ_ERR_NO_CONN;
 
 	return _mosquitto_send_subscribe(mosq, mid, false, sub, qos);
 }
 
 int mosquitto_unsubscribe(struct mosquitto *mosq, uint16_t *mid, const char *sub)
 {
-	if(!mosq || mosq->sock == INVALID_SOCKET) return 1;
+	if(!mosq) return MOSQ_ERR_INVAL;
+	if(mosq->core.sock == INVALID_SOCKET) return MOSQ_ERR_NO_CONN;
 
 	return _mosquitto_send_unsubscribe(mosq, mid, false, sub);
 }
@@ -260,14 +329,16 @@ int mosquitto_loop(struct mosquitto *mosq, int timeout)
 #endif
 	fd_set readfds, writefds;
 	int fdcount;
+	int rc;
 
-	if(!mosq || mosq->sock == INVALID_SOCKET) return 1;
+	if(!mosq) return MOSQ_ERR_INVAL;
+	if(mosq->core.sock == INVALID_SOCKET) return MOSQ_ERR_NO_CONN;
 
 	FD_ZERO(&readfds);
-	FD_SET(mosq->sock, &readfds);
+	FD_SET(mosq->core.sock, &readfds);
 	FD_ZERO(&writefds);
-	if(mosq->out_packet){
-		FD_SET(mosq->sock, &writefds);
+	if(mosq->core.out_packet){
+		FD_SET(mosq->core.sock, &writefds);
 	}
 	if(timeout >= 0){
 		local_timeout.tv_sec = timeout/1000;
@@ -286,58 +357,57 @@ int mosquitto_loop(struct mosquitto *mosq, int timeout)
 	}
 
 #ifndef WIN32
-	fdcount = pselect(mosq->sock+1, &readfds, &writefds, NULL, &local_timeout, NULL);
+	fdcount = pselect(mosq->core.sock+1, &readfds, &writefds, NULL, &local_timeout, NULL);
 #else
-	fdcount = select(mosq->sock+1, &readfds, &writefds, NULL, &local_timeout);
+	fdcount = select(mosq->core.sock+1, &readfds, &writefds, NULL, &local_timeout);
 #endif
 	if(fdcount == -1){
-		_mosquitto_log_printf(mosq, MOSQ_LOG_ERR, "Error in pselect: %s\n", strerror(errno));
-		return 1;
+		return 1; // FIXME what error to return?
 	}else{
-		if(FD_ISSET(mosq->sock, &readfds)){
-			if(mosquitto_loop_read(mosq)){
-				_mosquitto_socket_close(mosq);
-				if(mosq->state == mosq_cs_disconnecting){
+		if(FD_ISSET(mosq->core.sock, &readfds)){
+			rc = mosquitto_loop_read(mosq);
+			if(rc){
+				_mosquitto_socket_close(&mosq->core);
+				if(mosq->core.state == mosq_cs_disconnecting){
 					if(mosq->on_disconnect){
 						mosq->on_disconnect(mosq->obj);
 					}
-					return 0;
+					return MOSQ_ERR_SUCCESS;
 				}else{
-					fprintf(stderr, "Error in network read.\n");
-					return 1;
+					return rc;
 				}
 			}
 		}
-		if(FD_ISSET(mosq->sock, &writefds)){
-			if(mosquitto_loop_write(mosq)){
-				_mosquitto_socket_close(mosq);
-				if(mosq->state == mosq_cs_disconnecting){
+		if(FD_ISSET(mosq->core.sock, &writefds)){
+			rc = mosquitto_loop_write(mosq);
+			if(rc){
+				_mosquitto_socket_close(&mosq->core);
+				if(mosq->core.state == mosq_cs_disconnecting){
 					if(mosq->on_disconnect){
 						mosq->on_disconnect(mosq->obj);
 					}
-					return 0;
+					return MOSQ_ERR_SUCCESS;
 				}else{
-					fprintf(stderr, "Error in network write.\n");
-					return 1;
+					return rc;
 				}
 			}
 		}
 	}
 	mosquitto_loop_misc(mosq);
 
-	return 0;
+	return MOSQ_ERR_SUCCESS;
 }
 
 int mosquitto_loop_misc(struct mosquitto *mosq)
 {
-	if(!mosq) return 1;
+	if(!mosq) return MOSQ_ERR_INVAL;
 
 	_mosquitto_check_keepalive(mosq);
 	if(mosq->last_retry_check+1 < time(NULL)){
 		_mosquitto_message_retry_check(mosq);
 		mosq->last_retry_check = time(NULL);
 	}
-	return 0;
+	return MOSQ_ERR_SUCCESS;
 }
 
 int mosquitto_loop_read(struct mosquitto *mosq)
@@ -346,7 +416,8 @@ int mosquitto_loop_read(struct mosquitto *mosq)
 	ssize_t read_length;
 	int rc = 0;
 
-	if(!mosq || mosq->sock == INVALID_SOCKET) return 1;
+	if(!mosq) return MOSQ_ERR_INVAL;
+	if(mosq->core.sock == INVALID_SOCKET) return MOSQ_ERR_NO_CONN;
 	/* This gets called if pselect() indicates that there is network data
 	 * available - ie. at least one byte.  What we do depends on what data we
 	 * already have.
@@ -361,101 +432,104 @@ int mosquitto_loop_read(struct mosquitto *mosq)
 	 * After all data is read, send to _mosquitto_handle_packet() to deal with.
 	 * Finally, free the memory and reset everything to starting conditions.
 	 */
-	if(!mosq->in_packet.command){
+	if(!mosq->core.in_packet.command){
 		/* FIXME - check command and fill in expected length if we know it.
 		 * This means we can check the client is sending valid data some times.
 		 */
-#ifndef WIN32
-		read_length = read(mosq->sock, &byte, 1);
-#else
-		read_length = recv(mosq->sock, &byte, 1, 0);
-#endif
+		read_length = _mosquitto_net_read(&mosq->core, &byte, 1);
 		if(read_length == 1){
-			mosq->in_packet.command = byte;
+			mosq->core.in_packet.command = byte;
 		}else{
-			if(read_length == 0) return 1; /* EOF */
+			if(read_length == 0) return MOSQ_ERR_CONN_LOST; /* EOF */
 #ifndef WIN32
 			if(errno == EAGAIN || errno == EWOULDBLOCK){
 #else
 			if(WSAGetLastError() == WSAEWOULDBLOCK){
 #endif
-				return 0;
+				return MOSQ_ERR_SUCCESS;
 			}else{
-				return 1;
+				switch(errno){
+					case ECONNRESET:
+						return MOSQ_ERR_CONN_LOST;
+					default:
+						return 1;
+				}
 			}
 		}
 	}
-	if(!mosq->in_packet.have_remaining){
+	if(!mosq->core.in_packet.have_remaining){
 		/* Read remaining
 		 * Algorithm for decoding taken from pseudo code at
 		 * http://publib.boulder.ibm.com/infocenter/wmbhelp/v6r0m0/topic/com.ibm.etools.mft.doc/ac10870_.htm
 		 */
 		do{
-#ifndef WIN32
-			read_length = read(mosq->sock, &byte, 1);
-#else
-			read_length = recv(mosq->sock, &byte, 1, 0);
-#endif
+			read_length = _mosquitto_net_read(&mosq->core, &byte, 1);
 			if(read_length == 1){
-				mosq->in_packet.remaining_count++;
+				mosq->core.in_packet.remaining_count++;
 				/* Max 4 bytes length for remaining length as defined by protocol.
 				 * Anything more likely means a broken/malicious client.
 				 */
-				if(mosq->in_packet.remaining_count > 4) return 1;
+				if(mosq->core.in_packet.remaining_count > 4) return MOSQ_ERR_PROTOCOL;
 
-				mosq->in_packet.remaining_length += (byte & 127) * mosq->in_packet.remaining_mult;
-				mosq->in_packet.remaining_mult *= 128;
+				mosq->core.in_packet.remaining_length += (byte & 127) * mosq->core.in_packet.remaining_mult;
+				mosq->core.in_packet.remaining_mult *= 128;
 			}else{
-				if(read_length == 0) return 1; /* EOF */
+				if(read_length == 0) return MOSQ_ERR_CONN_LOST; /* EOF */
 #ifndef WIN32
 				if(errno == EAGAIN || errno == EWOULDBLOCK){
 #else
 				if(WSAGetLastError() == WSAEWOULDBLOCK){
 #endif
-					return 0;
+					return MOSQ_ERR_SUCCESS;
 				}else{
-					return 1;
+					switch(errno){
+						case ECONNRESET:
+							return MOSQ_ERR_CONN_LOST;
+						default:
+							return 1;
+					}
 				}
 			}
 		}while((byte & 128) != 0);
 
-		if(mosq->in_packet.remaining_length > 0){
-			mosq->in_packet.payload = malloc(mosq->in_packet.remaining_length*sizeof(uint8_t));
-			if(!mosq->in_packet.payload) return 1;
-			mosq->in_packet.to_process = mosq->in_packet.remaining_length;
+		if(mosq->core.in_packet.remaining_length > 0){
+			mosq->core.in_packet.payload = _mosquitto_malloc(mosq->core.in_packet.remaining_length*sizeof(uint8_t));
+			if(!mosq->core.in_packet.payload) return MOSQ_ERR_NOMEM;
+			mosq->core.in_packet.to_process = mosq->core.in_packet.remaining_length;
 		}
-		mosq->in_packet.have_remaining = 1;
+		mosq->core.in_packet.have_remaining = 1;
 	}
-	while(mosq->in_packet.to_process>0){
-#ifndef WIN32
-		read_length = read(mosq->sock, &(mosq->in_packet.payload[mosq->in_packet.pos]), mosq->in_packet.to_process);
-#else
-		read_length = recv(mosq->sock, &(mosq->in_packet.payload[mosq->in_packet.pos]), mosq->in_packet.to_process, 0);
-#endif
+	while(mosq->core.in_packet.to_process>0){
+		read_length = _mosquitto_net_read(&mosq->core, &(mosq->core.in_packet.payload[mosq->core.in_packet.pos]), mosq->core.in_packet.to_process);
 		if(read_length > 0){
-			mosq->in_packet.to_process -= read_length;
-			mosq->in_packet.pos += read_length;
+			mosq->core.in_packet.to_process -= read_length;
+			mosq->core.in_packet.pos += read_length;
 		}else{
 #ifndef WIN32
 			if(errno == EAGAIN || errno == EWOULDBLOCK){
 #else
 			if(WSAGetLastError() == WSAEWOULDBLOCK){
 #endif
-				return 0;
+				return MOSQ_ERR_SUCCESS;
 			}else{
-				return 1;
+				switch(errno){
+					case ECONNRESET:
+						return MOSQ_ERR_CONN_LOST;
+					default:
+						return 1;
+				}
 			}
 		}
 	}
 
 	/* All data for this packet is read. */
-	mosq->in_packet.pos = 0;
+	mosq->core.in_packet.pos = 0;
 	rc = _mosquitto_packet_handle(mosq);
 
 	/* Free data and reset values */
-	_mosquitto_packet_cleanup(&mosq->in_packet);
+	_mosquitto_packet_cleanup(&mosq->core.in_packet);
 
-	mosq->last_msg_in = time(NULL);
+	mosq->core.last_msg_in = time(NULL);
 	return rc;
 }
 
@@ -465,33 +539,35 @@ int mosquitto_loop_write(struct mosquitto *mosq)
 	ssize_t write_length;
 	struct _mosquitto_packet *packet;
 
-	if(!mosq || mosq->sock == INVALID_SOCKET) return 1;
+	if(!mosq) return MOSQ_ERR_INVAL;
+	if(mosq->core.sock == INVALID_SOCKET) return MOSQ_ERR_NO_CONN;
 
-	while(mosq->out_packet){
-		packet = mosq->out_packet;
+	while(mosq->core.out_packet){
+		packet = mosq->core.out_packet;
 
 		if(packet->command){
 			/* Assign to_proces here before remaining_length changes. */
 			packet->to_process = packet->remaining_length;
 			packet->pos = 0;
 
-#ifndef WIN32
-			write_length = write(mosq->sock, &packet->command, 1);
-#else
-			write_length = send(mosq->sock, &packet->command, 1, 0);
-#endif
+			write_length = _mosquitto_net_write(&mosq->core, &packet->command, 1);
 			if(write_length == 1){
 				packet->command = 0;
 			}else{
-				if(write_length == 0) return 1; /* EOF */
+				if(write_length == 0) return MOSQ_ERR_CONN_LOST; /* EOF */
 #ifndef WIN32
 				if(errno == EAGAIN || errno == EWOULDBLOCK){
 #else
 				if(WSAGetLastError() == WSAEWOULDBLOCK){
 #endif
-					return 0;
+					return MOSQ_ERR_SUCCESS;
 				}else{
-					return 1;
+					switch(errno){
+						case ECONNRESET:
+							return MOSQ_ERR_CONN_LOST;
+						default:
+							return 1;
+					}
 				}
 			}
 		}
@@ -507,37 +583,34 @@ int mosquitto_loop_write(struct mosquitto *mosq)
 				if(packet->remaining_length>0){
 					byte = byte | 0x80;
 				}
-#ifndef WIN32
-				write_length = write(mosq->sock, &byte, 1);
-#else
-				write_length = send(mosq->sock, &byte, 1, 0);
-#endif
+				write_length = _mosquitto_net_write(&mosq->core, &byte, 1);
 				if(write_length == 1){
 					packet->remaining_count++;
 					/* Max 4 bytes length for remaining length as defined by protocol. */
-					if(packet->remaining_count > 4) return 1;
+					if(packet->remaining_count > 4) return MOSQ_ERR_PROTOCOL;
 	
 				}else{
-					if(write_length == 0) return 1; /* EOF */
+					if(write_length == 0) return MOSQ_ERR_CONN_LOST; /* EOF */
 #ifndef WIN32
 					if(errno == EAGAIN || errno == EWOULDBLOCK){
 #else
 					if(WSAGetLastError() == WSAEWOULDBLOCK){
 #endif
-						return 0;
+						return MOSQ_ERR_SUCCESS;
 					}else{
-						return 1;
+						switch(errno){
+							case ECONNRESET:
+								return MOSQ_ERR_CONN_LOST;
+							default:
+								return 1;
+						}
 					}
 				}
 			}while(packet->remaining_length > 0);
 			packet->have_remaining = 1;
 		}
 		while(packet->to_process > 0){
-#ifndef WIN32
-			write_length = write(mosq->sock, &(packet->payload[packet->pos]), packet->to_process);
-#else
-			write_length = send(mosq->sock, &(packet->payload[packet->pos]), packet->to_process, 0);
-#endif
+			write_length = _mosquitto_net_write(&mosq->core, &(packet->payload[packet->pos]), packet->to_process);
 			if(write_length > 0){
 				packet->to_process -= write_length;
 				packet->pos += write_length;
@@ -547,9 +620,14 @@ int mosquitto_loop_write(struct mosquitto *mosq)
 #else
 				if(WSAGetLastError() == WSAEWOULDBLOCK){
 #endif
-					return 0;
+					return MOSQ_ERR_SUCCESS;
 				}else{
-					return 1;
+					switch(errno){
+						case ECONNRESET:
+							return MOSQ_ERR_CONN_LOST;
+						default:
+							return 1;
+					}
 				}
 			}
 		}
@@ -560,42 +638,42 @@ int mosquitto_loop_write(struct mosquitto *mosq)
 		}
 
 		/* Free data and reset values */
-		mosq->out_packet = packet->next;
+		mosq->core.out_packet = packet->next;
 		_mosquitto_packet_cleanup(packet);
-		free(packet);
+		_mosquitto_free(packet);
 
-		mosq->last_msg_out = time(NULL);
+		mosq->core.last_msg_out = time(NULL);
 	}
-	return 0;
+	return MOSQ_ERR_SUCCESS;
 }
 
 void mosquitto_connect_callback_set(struct mosquitto *mosq, void (*on_connect)(void *, int))
 {
-	if(mosq) mosq->on_connect = on_connect;
+	mosq->on_connect = on_connect;
 }
 
 void mosquitto_disconnect_callback_set(struct mosquitto *mosq, void (*on_disconnect)(void *))
 {
-	if(mosq) mosq->on_disconnect = on_disconnect;
+	mosq->on_disconnect = on_disconnect;
 }
 
 void mosquitto_publish_callback_set(struct mosquitto *mosq, void (*on_publish)(void *, uint16_t))
 {
-	if(mosq) mosq->on_publish = on_publish;
+	mosq->on_publish = on_publish;
 }
 
 void mosquitto_message_callback_set(struct mosquitto *mosq, void (*on_message)(void *, const struct mosquitto_message *))
 {
-	if(mosq) mosq->on_message = on_message;
+	mosq->on_message = on_message;
 }
 
 void mosquitto_subscribe_callback_set(struct mosquitto *mosq, void (*on_subscribe)(void *, uint16_t, int, const uint8_t *))
 {
-	if(mosq) mosq->on_subscribe = on_subscribe;
+	mosq->on_subscribe = on_subscribe;
 }
 
 void mosquitto_unsubscribe_callback_set(struct mosquitto *mosq, void (*on_unsubscribe)(void *, uint16_t))
 {
-	if(mosq) mosq->on_unsubscribe = on_unsubscribe;
+	mosq->on_unsubscribe = on_unsubscribe;
 }
 
